
import re


def split_ref(string):
    """ splits a string into (tag, id) """
    re_tag = re.compile('^(?P<tag>[a-zA-Z]+)[\s-]*(?P<pk>\d+)$')
    m = re_tag.search(string)
    if not m:
        raise ValueError("unable to split string '%s'" % (string,))

    return (m.group('tag').lower(), int(m.group('pk')))


def pretty_speed(value):
    if not value:
        return ''
    try:
        value = int(value)
        if value >= 1000000:
            return "%dT" % (value / 10**6)
        elif value >= 1000:
            return "%dG" % (value / 10**3)
        else:
            return "%dM" % value
    except ValueError:
        return value




import munge
import munge.codec.all
import munge.util
import os


default_conf_dir = '~/.peeringdb'


def default_config():
    conf = {
        'peeringdb': {
            'url': 'https://www.peeringdb.com/api',
            'user': '',
            'password': '',
            'timeout': 0,
        },
        'database': {
            'engine': 'sqlite3',
            'name': 'peeringdb.sqlite3',
            'host': '',
            'port': '',
            'user': '',
            'password': '',
        }
    }
    return conf.copy()


def get_config(conf_dir=default_conf_dir):
    if not conf_dir:
        return default_config()

    conf_path = os.path.expanduser(conf_dir)
    if not os.path.exists(conf_path):
        # only throw if not default
        if conf_dir != default_conf_dir:
            raise IOError("config dir not found at %s" % (conf_path,))

    data = default_config()
    config = munge.load_datafile('config', conf_path, default=None)
    if config:
        munge.util.recursive_update(data, config)
        data['__config_dir__'] = conf_path

    return data


def write_config(data, conf_dir=default_conf_dir, codec=None):
    if not codec:
        codec = munge.get_codec('yaml')()
    conf_dir = os.path.expanduser(conf_dir)
    if not os.path.exists(conf_dir):
        os.mkdir(conf_dir)

    codec.dump(data, open(os.path.join(conf_dir, 'config.' + codec.extensions[0]), 'w'))




import django
import os
from django.conf import settings
from django.core.management import call_command
from django.db import connection
import warnings

# lazy init for translations
_ = lambda s: s


def django_configure(cfg):
    if settings.configured:
        return

    db_fields = (
        'ENGINE',
        'HOST',
        'NAME',
        'PASSWORD',
        'PORT',
        'USER',
    )
    db = {}
    if 'database' in cfg:
        for k,v in cfg['database'].items():
            k = k.upper()
            if k in db_fields:
                db[k] = v

    else:
        db = {
            'ENGINE': 'sqlite3',
            'NAME': ':memory:',
        }

    if 'peeringdb' in cfg:
        extra = {
            'PEERINGDB_SYNC_URL': cfg['peeringdb'].get('url', ''),
            'PEERINGDB_SYNC_USERNAME': cfg['peeringdb'].get('user', ''),
            'PEERINGDB_SYNC_PASSWORD': cfg['peeringdb'].get('password', ''),
            'PEERINGDB_SYNC_ONLY': cfg['peeringdb'].get('sync_only', []),
            'PEERINGDB_SYNC_STRIP_TZ': True,
        }
    else:
        extra = {
            'PEERINGDB_SYNC_STRIP_TZ': True,
        }

    # open file reletive to config dir
    if '__config_dir__' in cfg:
        os.chdir(cfg['__config_dir__'])

    db['ENGINE'] = 'django.db.backends.' + db['ENGINE']

    settings.configure(
        INSTALLED_APPS=[
            'django_peeringdb',
        ],
        DATABASES={
            'default': db
        },
        DEBUG=False,
        TEMPLATE_DEBUG=True,
        LOGGING={
            'version': 1,
            'disable_existing_loggers': False,
            'filters': {
                'dev_warnings': {
                    }
                },
            'handlers': {
                'stderr': {
                    'level': 'DEBUG',
                    'class': 'logging.StreamHandler',
                    'filters': ['dev_warnings'],
                    },
                },
            'loggers': {
                '': {
                    'handlers': ['stderr'],
                    'level': 'DEBUG',
                    'propagate': False
                },
            },
        },

        USE_TZ=False,
        # add user defined iso code for Kosovo
        COUNTRIES_OVERRIDE={
            'XK': _('Kosovo'),
        },
        **extra
    )


class LocalDB(object):
    """
    Abstraction for a local database

    currently this is just django, to extend that, would just need to
    extend this interface
    """

    def __init__(self, cfg):
        self.cfg = cfg
        django_configure(cfg)
        django.setup()

        # get rid of django deprecation warnings
        # TODO fix log filters
        warnings.filterwarnings("ignore")

    def create(self):
        call_command('migrate', interactive=False)

    def drop_tables(self):
        """ drop tables this added """
        print("This command has been removed temporarily")

    def sync(self):
        self.create()
        call_command('pdb_sync', interactive=False)



from __future__ import print_function

import collections
import sys
from peeringdb.util import pretty_speed


class WhoisFormat(object):
    def __init__(self, fobj=sys.stdout):
        self.fobj = fobj

        self.display_names = {
            'fac_set': 'Facilities',
            }

    def mk_fmt(self, *widths):
        return '%-' + 's %-'.join(map(str, widths)) + 's'

    def mk_set_headers(self, data, columns):
        """ figure out sizes and create header fmt """
        columns = tuple(columns)
        lens = []

        for key in columns:
            value_len = max(len(str(each.get(key, ''))) for each in data)
            # account for header lengths
            lens.append(max(value_len, len(self._get_name(key))))

        fmt = self.mk_fmt(*lens)
        return fmt

    def _get_name(self, key):
        """ get display name for a key, or mangle for display """
        if key in self.display_names:
            return self.display_names[key]

        return key.capitalize()

    def _get_val(self, data, key):
        """ get value from a dict, format if necessary """
        return data.get(key, '')

    def _get_columns(self, data):
        """ get columns from a dict """
        return data.keys()

    def display_section(self, name):
        self._print(name)
        self._print('=' * len(name))
        self._print("")

    def display_headers(self, fmt, headers):
        self._print(fmt % headers)
        self._print(fmt % tuple('-' * len(x) for x in headers))

    def display_set(self, typ, data, columns):
        """ display a list of dicts """
        self.display_section("%s (%d)" % (self._get_name(typ), len(data)))
        headers = tuple(map(self._get_name, columns))
        fmt = self.mk_set_headers(data, columns)
        self.display_headers(fmt, headers)

        for each in data:
            row = tuple(self._get_val(each, k) for k,v in each.items())
            self._print(fmt % row)

        self._print("\n")

    def display_field(self, fmt, obj, field, display=None):
        if not display:
            display = self._get_name(field)
        self._print(fmt % (display, obj[field]))

    def check_set(self, data, name):
        if data.get(name, None):
            if hasattr(self, 'print_' + name):
                getattr(self, 'print_' + name)(data[name])

    def print_net(self, data):
        self.display_section("Network Information")
        fmt = "%-21s: %s"
        self.display_field(fmt, data, 'name', 'Name')
        self.display_field(fmt, data, 'asn', 'Primary ASN')
        self.display_field(fmt, data, 'aka', 'Also Known As')
        self.display_field(fmt, data, 'website', 'Website')
        self.display_field(fmt, data, 'irr_as_set', 'IRR AS-SET')
        self.display_field(fmt, data, 'info_type', 'Network Type')
        self.display_field(fmt, data, 'info_prefixes6', 'Approx IPv6 Prefixes')
        self.display_field(fmt, data, 'info_prefixes4', 'Approx IPv4 Prefixes')
        self.display_field(fmt, data, 'looking_glass', 'Looking Glass')
        self.display_field(fmt, data, 'route_server', 'Route Server')
        self.display_field(fmt, data, 'created', 'Created at')
        self.display_field(fmt, data, 'updated', 'Updated at')
        self._print("\n")

        self.display_section("Peering Policy Information")
        self.display_field(fmt, data, 'policy_url', 'URL')
        self.display_field(fmt, data, 'policy_general', 'General Policy')
        self.display_field(fmt, data, 'policy_locations', 'Location Requirement')
        self.display_field(fmt, data, 'policy_ratio', 'Ratio Requirement')
        self.display_field(fmt, data, 'policy_contracts', 'Contract Requirement')
        self._print("\n")

        self.check_set(data, 'poc_set')
        self.check_set(data, 'netixlan_set')
        self.check_set(data, 'netfac_set')

    def print_poc_set(self, data):
        self.display_section("Contact Information")
        fmt = self.mk_fmt(6, 20, 15, 20, 14)
        hdr = ('Role', 'Name', 'Email', 'URL', 'Phone')
        self.display_headers(fmt, hdr)
        for poc in data:
            self._print(fmt % (poc.get('role', ''), poc.get('name', ''), poc.get('email', ''), poc.get('url', ''), poc.get('phone', '')))

        for poc in data:
            self._print(fmt % (poc.get('role', ''), poc.get('name', ''), poc.get('email', ''), poc.get('url', ''), poc.get('phone', '')))

        self._print("\n")

    def print_netfac_set(self, data):
        self.display_section("Private Peering Facilities (%d)" % len(data))
        fmt = self.mk_fmt(51, 8, 15, 2)
        hdr = ('Facility Name', 'ASN', 'City', 'CO')
        self.display_headers(fmt, hdr)
        for each in data:
            self._print(fmt % (each.get('name', each.get('id')), each.get('local_asn', ''), each.get('city', ''), each.get('country', '')))
        self._print("\n")

    def print_netixlan_set(self, data):
        self.display_section("Public Peering Points (%d)" % len(data))
        fmt = self.mk_fmt(36, 8, 27, 5)
        hdr = ('Exchange Point', 'ASN', 'IP Address', 'Speed')
        self.display_headers(fmt, hdr)
        for ix in data:
            if ix.get('ipaddr4', None):
                self._print(fmt % (ix.get('name', ix.get('ixlan_id')), ix['asn'], ix['ipaddr4'], pretty_speed(ix['speed'])))
            if ix.get('ipaddr6', None):
                if ix.get('ipaddr4', None):
                    self._print(fmt % ('', '', ix['ipaddr6'], ''))
                else:
                    self._print(fmt % (ix['name'], ix['asn'], ix['ipaddr6'], pretty_speed(ix['speed'])))
        self._print("\n")

    def _print(self, *args):
        """ internal print to self.fobj """
        string = u" ".join(args) + '\n'
        self.fobj.write(string.encode('utf8'))

    def print(self, typ, data):
        """ *deprecated* - use display() """
        return self.display(typ, data)

    def display(self, typ, data):
        """ display section of typ with data """
        if hasattr(self, 'print_' + typ):
            getattr(self, 'print_' + typ)(data)

        elif not data:
            self._print("%s: %s" % (typ, data))

        elif isinstance(data, collections.Mapping):
            self._print("\n", typ)
            for k,v in data.items():
                self.print(k, v)

        elif isinstance(data, (list, tuple)):
            # tabular data layout for lists of dicts
            if isinstance(data[0], collections.Mapping):
                self.display_set(typ, data, self._get_columns(data[0]))
            else:
                for each in data:
                    self.print(typ, each)

        else:
            self._print("%s: %s" % (typ, data))

        self.fobj.flush()




import munge.util
from peeringdb import config
import twentyc.rpc


class PeeringDB(twentyc.rpc.RestClient):
    def __init__(self, **kwargs):
        """
        options:
            conf_dir : directory to load config from
            url      : URL to connect to
            user     : user to connect as
            password : password to use
            timeout  : timeout to fail after
        """
        # try to load config
        cfg = config.get_config(conf_dir=kwargs.get('conf_dir', config.default_conf_dir))
        self.config = cfg['peeringdb']
        # override config with kwargs
        munge.util.recursive_update(self.config, kwargs)
        super(PeeringDB, self).__init__(**self.config)

    def asn(self, pk):
        return self.all('net', asn=pk, depth=2)

    def ixnets(self, pk):
        return self.all('net', ix_id__in=pk, depth=2)

    def whois(self, typ, pk, **kwargs):
        if typ == 'as':
            return ('net', self.asn(pk))
        elif typ == 'ixnets':
            return ('net', self.ixnets(pk))
        else:
            return (typ, self.get(typ, pk, **kwargs))




from __future__ import print_function

import subprocess

import click
import munge
import munge.codec
import munge.codec.all
import peeringdb
from peeringdb import client
from peeringdb import util
from peeringdb.whois import WhoisFormat
from pkg_resources import resource_stream
import sys


def install_deps(deps, quiet=True):
    for pkg in deps:
        subprocess.check_call([sys.executable, '-m', 'pip', 'install', pkg])


def get_deps(typ):
    """ get deps from requirement file of specified type """
    deps = []
    filename = 'deps/requirements-%s.txt' % typ
    with resource_stream("peeringdb", filename) as req:
        for line in req:
            deps.append(line.strip())
    return deps


def dict_prompt(data, key, default=''):
    data[key] = click.prompt(key, default=data.get(key, default))


def db_prompt(data):
    for k in ('host', 'port', 'name', 'user', 'password'):
        dict_prompt(data, k)


def cb_list_codecs(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(', '.join(munge.codec.list_codecs()))
    ctx.exit()


@click.group()
@click.version_option()
@click.option('--list-codecs', help='list available codecs',
    is_flag=True, callback=cb_list_codecs,
    expose_value=False, is_eager=True)
def cli():
    """
    PeeringDB
    """
    pass


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
def conf_write(config):
    """ write config file with defaults """
    cfg = peeringdb.config.get_config(config)
    peeringdb.config.write_config(cfg, config)
    print("config written to %s" % (config))
    return 0


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
@click.option('--output-format', default='yaml', help='output data format')
def conf_dump(config, output_format):
    """ output current config """
    cfg = peeringdb.config.get_config(config)
    codec = munge.get_codec(output_format)()
    codec.dump(cfg, sys.stdout)
    return 0


@cli.command()
@click.option('--config', help='config directory',
    envvar='PEERINGDB_HOME',
    default='~/.peeringdb', prompt='config directory')
@click.option('--database', help='database type for local sync',
    type=click.Choice(['none', 'mysql', 'sqlite3']),
    default='none',
    prompt='database type for local sync (mysql,sqlite3)')
def configure(config, database):
    """ configure peeringdb """
    cfg = peeringdb.config.get_config(config)
    db = {}
    if database != 'none':
        print("enter the database config, blank for engine defaults")
        db['engine'] = database
        if database == 'mysql':
            db_prompt(db)
        elif database == 'sqlite3':
            db['name'] = click.prompt('sqlite filename', default='peeringdb.sqlite3')

    cfg['database'] = db
    peeringdb.config.write_config(cfg, config)
    return 0


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
def depcheck(config):
    """ check for dependencies, install if necessary """
    cfg = peeringdb.config.get_config(config)
    engine = cfg.get('database', {}).get('engine')
#    if engine == 'mysql':
#    elif engine == 'sqlite3':
    install_deps(get_deps(engine))
    return 0


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
@click.option('--depth', default=2, help='how many levels of nested objects to fetch')
@click.option('--output-format', default='yaml', help='output data format')
@click.argument('poids', nargs=-1)
def get(config, depth, output_format, poids):
    """ get an object from peeringdb """
    pdb = client.PeeringDB(conf_dir=config)
    codec = munge.get_codec(output_format)()

    for poid in poids:
        (typ, pk) = util.split_ref(poid)
        data = pdb.get(typ, pk, depth=depth)
        codec.dump(data, sys.stdout)


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
@click.argument('poids', nargs=-1)
def whois(config, poids):
    """ simulate a whois lookup

        supports

            as<ASN> : query by AS

            ixlans<net id> : query networks on an IX
    """
    pdb = client.PeeringDB(conf_dir=config)
    fmt = WhoisFormat()

    for poid in poids:
        (typ, pk) = util.split_ref(poid)
        (typ, data) = pdb.whois(typ, pk)
        fmt.print(typ, data[0])


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
def sync(config):
    """ synchronize to a local database """
    # import here until the db is properly abstracted
    from peeringdb.localdb import LocalDB

    cfg = peeringdb.config.get_config(config)
    db = LocalDB(cfg)
    db.sync()
    return 0


@cli.command()
@click.option('--config', envvar='PEERINGDB_HOME', default='~/.peeringdb')
def drop_tables(config):
    """ drop all peeringdb tables NOTE this will delete data """
    # import here until the db is properly abstracted
    from peeringdb.localdb import LocalDB

    cfg = peeringdb.config.get_config(config)
    db = LocalDB(cfg)
    db.drop_tables()
    return 0




from setuptools import setup

version = open('facsimile/VERSION').read().strip()
requirements = open('facsimile/requirements.txt').read().split("\n")
test_requirements = open('facsimile/requirements-test.txt').read().split("\n")

setup(
    name='peeringdb',
    version=version,
    author='PeeringDB',
    author_email='support@peeringdb.com',
    description='peeringdb client and interface',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
    ],
    packages=[
        'peeringdb',
    ],
    package_data={'peeringdb': [
        'deps/requirements*.txt',
    ]},
    url='https://github.com/peeringdb/peeringdb-py',
    download_url='https://github.com/peeringdb/peeringdb-py/%s' % version,
    include_package_data=True,
    install_requires=requirements,
    test_requires=test_requirements,
    entry_points={
        'console_scripts': [
            'peeringdb=peeringdb.cli:cli',
        ]
    },
    zip_safe=True,
)

